
#ifndef GLWIDGET_H
#define GLWIDGET_H

#include <QOpenGLWidget>
#include <QtOpenGL>
#include "s3d.h"
#include <vector>
#include <string>
#include "exprtk.hpp"

typedef exprtk::symbol_table<double> symbol_table_t;
typedef exprtk::expression<double> expression_t;
typedef exprtk::parser<double> parser_t;
typedef exprtk::parser_error::type error_t;

typedef struct{
    double x, y, z;
}Xyz;

typedef struct{
    int p[3];
}Triangle;

typedef struct{
    Xyz p[8];
   double val[8];
}Gridcell;

class GLWidget : public QOpenGLWidget
{
public:
    explicit GLWidget(QWidget *parent = 0);
    ~GLWidget();
    void resizeGL(int width, int height);
    int load_scene_file(const char *name);
    void exampleMarching(int type, int resolution, int *nv, int *nt, double minX, double minY, double minZ, double maxX, double maxY, double maxZ, double fromX,
                         double fromY, double fromZ, double fov);
    void renderCloud();
    bool parseExpression();
    QSize sizeHint() const;

    std::string expression_str;

protected:
    void initializeGL();
    void paintGL();

    void keyPressEvent(QKeyEvent *event);

    void dragEnterEvent(QDragEnterEvent *event);
    void dragMoveEvent(QDragMoveEvent *event);
    void dragLeaveEvent(QDragLeaveEvent *event);
    void dropEvent(QDropEvent *event);

    void mousePressEvent(QMouseEvent *event);
    void mouseReleaseEvent(QMouseEvent *event);
    void mouseMoveEvent(QMouseEvent *event);
    void timerEvent(QTimerEvent *event);

private:
    QVector3D projectToSphere(double x, double y);
    Matrix4 quatToMatrix(QQuaternion &quat);
    double functionXyz(double x, double y, double z, int type);
    double evaluateExpression(double x, double y, double z);

    expression_t expression;
    symbol_table_t symbol_table;

    Scene *scene, *auxScene;

    Matrix4 m;
    bool rotating, translating;
    double radiusOfSphere;
    int xpos, ypos;
    QQuaternion quatStart, quatCurrent;
    QVector3D vecStart, vecTransCenter, centerOfScreen, vecCurrent;
};

#endif // GLWIDGET_H
